#!/bin/bash
###################EXPERIMENTAL PURPOSES ONLY####################
#                                                               #
# This script removes unnecessary packages and files needed for #
# a thin NeXtMidas no-graphical build                           #
#                                                               #
# The thin build is designed to provide NeXtMidas Java library  #
# code access and also NeXtMidas commands related to BLUE file  #
# processing and needed to support a standard NeXtMidas Build   #
#                                                               #
# Environment Variables:                                        #
#   NMROOT - Current NeXtMidas distribution                     #
#                                                               #
# Author: Zachary Matthews                                      #
# Since:  NeXtMidas 4.0.0                                       #
#                                                               #
#################################################################

######################################################################
# Separate lib-libg graphical dependencies and lucene jar dependency #
######################################################################
sh $NMROOT/swap/swap.sh nlucene splitliblibg

############################################################################################
# Modify the Environment intrinsic to use nxm.sys.lib.Theme in place of nxm.sys.libg.Theme #
############################################################################################
sed -i 's/import nxm\.sys\.libg\.Theme;/import nxm\.sys\.lib\.Theme;/' $NMROOT/nxm/sys/intr/Environment.java

##################################################################
# Switch to the "thin" version of commands.cnf & test_config.tbl #
##################################################################
cp $NMROOT/swap/thinNXM/commands.cnf $NMROOT/nxm/sys/cfg/commands.cnf
cp $NMROOT/swap/thinNXM/test_config.tbl $NMROOT/nxm/sys/test/test_config.tbl

####################################
# Remove unnecessary full packages #
####################################

# Remove NeXtMidas option tree packages that do not include sys 
rm -rf $NMROOT/nxm/audio/
rm -rf $NMROOT/nxm/database/
rm -rf $NMROOT/nxm/dsp/
rm -rf $NMROOT/nxm/hwc/
rm -rf $NMROOT/nxm/net/
rm -rf $NMROOT/nxm/nettest/
rm -rf $NMROOT/nxm/ucl/
rm -rf $NMROOT/nxm/vnc/
rm -rf $NMROOT/nxm/xbc/

# Remove unnecessary nxm.sys packages
rm -rf $NMROOT/nxm/sys/host/
rm -rf $NMROOT/nxm/sys/libg/
rm -rf $NMROOT/nxm/sys/libh/
rm -rf $NMROOT/nxm/sys/libxm/
rm -rf $NMROOT/nxm/sys/osgi/

########################################################
# Remove unnecessary classes under nxm.sys.net package #
########################################################

# nxm.sys.net classes to keep
sysNetClassesToKeep=(NetUtil)

# Remove all nxm.sys.net classs except the ones being kept 
for sysNetFile in $NMROOT/nxm/sys/net/* ;
do
  for classToKeep in ${sysNetClassesToKeep[@]} ;
  do
    if [[ $sysNetFile =~ $classToKeep ]] ; then
      continue 2
    fi
  done

  rm -f $sysNetFile
done


################################################################
# Remove unnecessary unit tests and update unittest_config.tbl #
################################################################

# remove selective lib tests that have libg dependencies
junitLibTestsToRemove=(CommandTest DatagramDataTest GPrimitiveTest RTPICEPacketsCacheTest RTPHeaderTest)
for junitLibTest in ${junitLibTestsToRemove[@]} ;
do
  rm -rf $NMROOT/nxm/sys/test/junit/lib/$junitLibTest.*
done

# remove all tests in packages in which corresponding source packages were removed
rm -rf $NMROOT/nxm/sys/test/junit/libg/*
rm -rf $NMROOT/nxm/sys/test/junit/net/*

# all current primitive tests get removed
rm -rf $NMROOT/nxm/sys/test/junit/prim/*

# modify unittest_config.tbl to exclude junit tests that were removed above
sed -i 's/\(^\(libg\|net\|prim\).*\)/!\1/' $NMROOT/nxm/sys/test/junit/unittest_config.tbl
sed -i 's/\(^lib.\(CommandTest\|DatagramDataTest\|GPrimitiveTest\|RTPICEPacketsCacheTest\|RTPHeaderTest\)\)/!\1/' $NMROOT/nxm/sys/test/junit/unittest_config.tbl

##############################################################################
# Remove unnecessary primitives, intrinsics, and associated macro test cases #
##############################################################################
sh $NMROOT/swap/thinNXM/remove_cmds_test_cases.sh

###########################################################
# Remove unnecessary additional non-junit java test files #
###########################################################

# remove these test files 
miscJavaTestFilesToRemove=(*plot* *Plot* testbeanpipehangbean test_widget_wrapper test_panel_jsetup test_panel_jsetup_no_pane0 test_panel_jsetup_wb test_panel_jsetup_jpanel test_list_jsetup test_dragbox_highres_fft test_jnuanes TestLayerDrawable TestNeXtMidas testprescalerhelper)

# Remove source files and other files directly associated with source files with these file extensions
fileExtensionsToRemove=(java class form html)

# Remove all test files having to do with plot with specified file extensions to remove
for javaTestFile in ${miscJavaTestFilesToRemove[@]} ;
do
  for fileExt in ${fileExtensionsToRemove[@]} ;
  do
    rm -f $NMROOT/nxm/sys/test/$javaTestFile.$fileExt
  done
done

# Remove these macro tests since they depend on the specified Java files mentioned above
miscMacroTestFilesToRemove=(test_midasbean test_dragbox_highres_fft test_jnuanes test_convert test_datalist_256ascii test_nextmidas)

for macroTestFile in ${miscMacroTestFilesToRemove[@]} ;
do
  rm -f $NMROOT/nxm/sys/test/$macroTestFile.m*
done

# Remove this script since it depends on TestNeXtMidas.java
miscScripts=runNativeCodeJar.sh



